@@ -37,9 +37,6 @@ define(['jquery', 'core/log', 'core/str', 'core/templates', 'core/notification']
         UP: 38
     };
 
-    /** @var {Number} closeSuggestionsTimer - integer used to cancel window.setTimeout. */
-    var closeSuggestionsTimer = null;
-
     /**
      * Make an item in the selection list "active".
      *
@@ -371,8 +368,12 @@ define(['jquery', 'core/log', 'core/str', 'core/templates', 'core/notification']
      * @param {Object} options Original options for this autocomplete element.
      * @param {Object} state State variables for this autocomplete element.
      * @param {JQuery} originalSelect The JQuery object matching the hidden select list.
+     * @param {boolean} markDirty Should the form be marked dirty after this update?
      */
-    var updateSelectionList = function(options, state, originalSelect) {
+    var updateSelectionList = function(options, state, originalSelect, markDirty) {
+        if (markDirty === undefined) {
+            markDirty = true;
+        }
         // Build up a valid context to re-render the template.
         var items = [];
         var newSelection = $(document.getElementById(state.selectionId));
@@ -405,6 +406,9 @@ define(['jquery', 'core/log', 'core/str', 'core/templates', 'core/notification']
         }).fail(notification.exception);
         // Because this function get's called after changing the selection, this is a good place
         // to trigger a change notification.
+        if (typeof M.core_formchangechecker != 'undefined' && markDirty) {
+            M.core_formchangechecker.set_form_changed({target: originalSelect});
+        }
         originalSelect.change();
     };
 
@@ -568,15 +572,12 @@ define(['jquery', 'core/log', 'core/str', 'core/templates', 'core/notification']
                 createItem(options, state, originalSelect);
             }
         });
-        inputElement.on('blur focus', function(e) {
-            // We may be blurring because we have clicked on the suggestion list. We
-            // dont want to close the selection list before the click event fires, so
-            // we have to delay.
-            if (closeSuggestionsTimer) {
-                window.clearTimeout(closeSuggestionsTimer);
-            }
-            closeSuggestionsTimer = window.setTimeout(function() {
-                if (e.type == 'blur') {
+        inputElement.on('blur', function() {
+            window.setTimeout(function() {
+                // Get the current element with focus.
+                var focusElement = $(document.activeElement);
+                // Only close the menu if the input hasn't regained focus.
+                if (focusElement.attr('id') != inputElement.attr('id')) {
                     if (options.tags) {
                         createItem(options, state, originalSelect);
                     }
@@ -589,9 +590,6 @@ define(['jquery', 'core/log', 'core/str', 'core/templates', 'core/notification']
             arrowElement.on('click', function() {
                 // Prevent the close timer, or we will open, then close the suggestions.
                 inputElement.focus();
-                if (closeSuggestionsTimer) {
-                    window.clearTimeout(closeSuggestionsTimer);
-                }
                 // Show the suggestions list.
                 updateSuggestions(options, state, inputElement.val(), originalSelect);
             });
@@ -732,6 +730,7 @@ define(['jquery', 'core/log', 'core/str', 'core/templates', 'core/notification']
             var context = $.extend({}, options, state);
             context.options = suggestions;
             context.items = [];
+            context.class = originalSelect.attr('class');
 
             var renderInput = templates.render('core/form_autocomplete_input', context);
             var renderDatalist = templates.render('core/form_autocomplete_suggestions', context);
@@ -765,7 +764,7 @@ define(['jquery', 'core/log', 'core/str', 'core/templates', 'core/notification']
                     });
                 }
                 // Show the current values in the selection list.
-                updateSelectionList(options, state, originalSelect);
+                updateSelectionList(options, state, originalSelect, false);
             });
         }
     };
